local DC = require "supertable.declare"
local CONF = require "supertable.core"
local pt = require "pl.pretty"

local ain = require "soc.ain"

local _M = {
	config_file = "/data/configs/aenc_config.conf"
}

local codec_template = DC.OBJ{
	source = DC.OBJ{
		audioSource = DC.STR("hdmi"),
		enWorkmode = DC.STR("I2S_SLAVE"), --NOTE: 3521D 不能改，硬件决定
		enSamplerate = DC.INT(48000),
		enBitwidth = DC.INT(16),
		enSoundmode = DC.STR("STEREO"),
		u32EXFlag = DC.INT(0),
		u32FrmNum = DC.INT(30),
		u32PtNumPerFrm = DC.INT(1024),
		u32ClkChnCnt = DC.INT(2),
		u32ChnCnt = DC.INT(2),
		u32ClkSel = DC.INT(0)
	},
	audioEncode = DC.ARR(DC.OBJ{
		vqeVolume = DC.INT(0),
		resample = DC.STR("fastest"),
		channel = DC.STR("STEREO"),
		channels = DC.INT(0),
		sampleSize = DC.INT(1024),
		sampling = DC.INT(48000),
		aacType = DC.STR("AAC"),
		aacFormat = DC.STR("RAW"),
		bitrate = DC.INT(64000),
		sampleWidth = DC.INT(16),
		g711Law = DC.STR("PCMU"),
		channelMap = DC.INT(1),
	})
}: __changed( function(conf)
	conf:TAB_SAVE()
end)

local default = {
	source = {
		audioSource = "hdmi",
		enWorkmode = "I2S_SLAVE", --NOTE: 3521D 不能改，硬件决定
		enSamplerate = 48000,
		enBitwidth = 16,
		-- enSoundmode = "MONO",
		enSoundmode = "STEREO",
		u32EXFlag = 0,
		u32FrmNum = 30,
		u32PtNumPerFrm = 1024,
		u32ClkChnCnt = 2,
		u32ChnCnt = 2,
		u32ClkSel = 0
	}, -- 音频输入源
    audioEncode = {
		{
			name = "Channel-1",
			vqeVolume = 0,
			resample = "fastest",
			sampleSize = 1024,
			channels = 2,
			sampling = 48000,
			aacType = "AAC",
			aacFormat = "RAW",
			bitrate = 64000,
			g711Law = "PCMU",
			channelMap = 1,
		},
		{
			name = "Channel-2",
			vqeVolume = 0,
			resample = "fastest",
			sampleSize = 480,
			channels = 1,
			sampling = 8000,
			aacType = "G711",
			aacFormat = "RAW",
			bitrate = 64000,
			g711Law = "PCMU",
			channelMap = 1,
		}
    },
	
}

--------------------------------------------
function _M.Load()
	local conf = CONF.Create({}, codec_template)
	conf:TAB_STORAGE( "file", { file =  _M.config_file } )
	local _, r = conf:TAB_LOAD()
	if not r then --Try load a fail-default configure
		conf:TAB_BIND( default )
	end

	conf:TAB_RESET_CHANGE()
	return conf
end

-------------------------------------------------------
return _M