local DC = require "supertable.declare"
local CONF = require "supertable.core"
local mod_OSD

local _M = {
	config_file = "/data/configs/osd.conf"
}
--------------------------------------------------------

local function on_ConfigChanged(cfg)
	mod_OSD = mod_OSD or require "mod_OSD"
	if not mod_OSD.initialized then
		mod_OSD.ConstructOSDs()
	else
		mod_OSD.UpdateOSDConfig()
	end
end

--------------------------------------------------------
local osd_template = DC.OBJ({
	channels = DC.ARR( DC.OBJ{
		config = DC.OBJ{
			enable = DC.BOOL(true),
			gridW = DC.INT(24),
			gridH = DC.INT(10),
			gridLineSpace = DC.NUM(0.5),
			gridShiftX = DC.NUM(1.5),
			gridShiftY = DC.NUM(0.5)
		},

		items = DC.ARR( DC.OBJ{
			enable = DC.BOOL(true),
			type = DC.STR( "none" ),
			position = DC.STR( "left-top" ),
			customX = DC.NUM(0),
			customY = DC.NUM(0),
			size = DC.INT(0),
			style = DC.STR("normal"),
			outline = DC.INT(1),
			color = DC.INT(1),
			outlineColor = DC.INT(0),
			text = DC.STR(""),
		})
	})
}) : __changed( function(conf)
	on_ConfigChanged(conf)
	conf:TAB_SAVE()
end)

-------------------------------------------------------
local failDefault = {
	channels = {
		[1] = {
			config = {
				enable = true,
				gridShiftX = 1.5,
				gridShiftY = 0.5,
				gridLineSpace = 0.5,
				gridW = 24,
				gridH = 10,
			},
			items = {
			}
		},
		[2] = {
			config = {
				enable = true,
				gridShiftX = 1.5,
				gridShiftY = 0.5,
				gridLineSpace = 0.5,
				gridW = 24,
				gridH = 10,
			},
			items = {
			}
		}
	}
}
-------------------------------------------------------
function _M.Load()
	local conf = CONF.Create({}, osd_template)
	conf:TAB_STORAGE( "file", { file =  _M.config_file } )
	local _, r = conf:TAB_LOAD()
	if not r then --Try load a fail-default configure
		conf:TAB_BIND( failDefault )
	end

	conf:TAB_RESET_CHANGE()
	return conf
end

-------------------------------------------------------

return _M


