require "himpp"
local _dbg = require "coaf.debug"

local SYS = HiMPP.SYS
local AI = HiMPP.AI
local ACODEC = HiMPP.ACODEC
local features = "HI3521D"

local _M = {
	MAX_CHANNELS = 2,
	aDev = "/dev/acodec",
	aCodec = nil,
	channels = {}
}

--note only HI3536 support ACODEC
local function Init_3536(resetCodec)
	if _M.aCodec then
		_dbg.Warn( "AIN", "The audio in module is already initialized.")
		return
	end

	_M.aCodec = ACODEC.CreateInnerCodec( _M.aDev )
	if not _M.aCodec then
		_dbg.Error( "AIN", "Fail to create audio Codec!" )
		return
	end

	if _M.aCodec:Open() < 0 then
		_dbg.Error( "AIN", "Fail to open audio Codec!" )
		_M.aCodec = nil
		return
	end

	if resetCodec then
		_M.aCodec:Reset()
	end

	_M.aCodec:Close()
end

local function Init_3521D()

end

function _M.Cleanup()
	for _, chanObj in pairs(_M.channels) do
		if chanObj then
			chanObj:Destroy()
		end
	end
	_M.channels = {}

	if _M.aCodec then
		_M.aCodec:Open()
		_M.aCodec:Reset()
		_M.aCodec:Close()
		_M.aCodec = nil
	end
end

--------------------------------------------------

local channel_meta = {}

-- Options:
-- {
-- 		channel = "MONO"/"STEREO",
-- 		sampleRate = ...,
-- 		sampleWidth = 8/16/24,
-- 		sampleSize = 256/512/...
-- }

function channel_meta:SetAudioOption( opts )
	local changed = false
	local srChanged = false
	if opts.enSoundmode and opts.enSoundmode ~= self.aiPubAttr.enSoundmode then
		self.aiPubAttr.enSoundmode = opts.enSoundmode
		changed = true
	end

	if opts.enSamplerate and opts.enSamplerate ~= self.aiPubAttr.enSamplerate then
		self.aiPubAttr.enSamplerate = opts.enSamplerate
		changed = true
		srChanged = true
	end

	if opts.enBitwidth and opts.enBitwidth ~= self.aiPubAttr.enBitwidth then
		self.aiPubAttr.enBitwidth = opts.enBitwidth
		changed = true
	end

	if opts.u32PtNumPerFrm and opts.u32PtNumPerFrm ~= self.aiPubAttr.u32PtNumPerFrm then
		self.aiPubAttr.u32PtNumPerFrm = opts.u32PtNumPerFrm
		changed = true
	end

	if opts.enSoundmode and opts.enSoundmode ~= self.aiPubAttr.enSoundmode then
		self.aiPubAttr.enSoundmode = opts.enSoundmode
		changed = true
	end

	if srChanged and _M.aCodec then
		_M.aCodec:Open()
		_M.aCodec:SetSampleRate( self.aiPubAttr.enSamplerate )
		_M.aCodec:Close()
	end

	if changed and self.initialized then
		-- if self.started then
		-- end
		self.aiChannel:Disable()
		self.aiDev:Disable()

		local r, err = self.aiDev:SetPubAttr( self.aiPubAttr )
		if not r then
			_dbg.Error( "AIN", "Fail to configure audio IN device, error:", err )
			local pt = require "pl.pretty"
			pt.dump(self.aiPubAttr)

			return
		end

		-- if self.VqeEnable then
		-- 	self.VqeEnable = nil
		-- 	self.aiChannel:DisableVqe()
		-- end

		self.aiDev:Enable()
		if self.started then
			self.aiChannel:Enable()
			if self.aiPubAttr.enSoundmode == "MONO" then
				-- self.aiChannel:SetVqeAttr(self.vqeAttr)
				-- self.aiChannel:EnableVqe()
				-- self.VqeEnable = true
			end
		end
	end
	return true
end

function channel_meta:SetVqeVolume(vol)
	if self.VqeEnable  then
		-- self.aiChannel:SetVeqVolume(vol)--NOTE:方法名是底层写错了，这里将错就错
	end
end

function channel_meta:GetVqeVolume()
	if self.VqeEnable  then
		-- return self.aiChannel:GetVeqVolume(vol)--NOTE:方法名是底层写错了，这里将错就错
	end
end

function channel_meta:Init()
	if self.initialized then
		return
	end

	if _M.aCodec then
		_M.aCodec:Open()
		if _M.aCodec:SelectSource( self.aiSource ) ~= 0 then
			_dbg.Warn( "AIN", "Fail to select audio source:", self.aiSource )
		end

		if _M.aCodec:SetSampleRate( self.aiPubAttr.enSamplerate ) ~= 0 then
			_dbg.Warn( "AIN", "Fail to set audio Codec chip sample rate to ", self.aiPubAttr.enSamplerate )
		end

		_M.aCodec:Close()
	end

	self.aiChannel = self.aiDev:GetChannel(0)
	self.aiChannel:Disable()
	self.aiDev:Disable()

	local r, err = self.aiDev:SetPubAttr( self.aiPubAttr )
	if not r then
		_dbg.Error( "AIN", "Fail to configure audio IN device, error:", err )
		local pt = require "pl.pretty"
		pt.dump(self.aiPubAttr)
		return
	end

	r = self.aiDev:Enable()
	if r ~= 0 then
		_dbg.Error( "AIN", "Fail to enable audio IN device! error code:", string.format("%08X",r) )
		return
	end

	
	self.initialized = true
end

function channel_meta:Destroy()
	if self.initialized then
		self.aiChannel:Disable()
		self.aiChannel = nil
		self.aiDev:Disable()
		self.initialized = false
	end
end

function channel_meta:Start()
	if not self.initialized then
		self:Init()
	end

	if self.initialized then
		self.aiChannel:Enable()
		self.started = true
		-- local r, old = self.aiChannel:GetParam()
		-- if not r then
		-- 	_dbg.Error( "AIN", old )
		-- 	return
		-- end

		-- if old and (not old.u32UsrFrmDepth or old.u32UsrFrmDepth ~= 30) then
			local r, err = self.aiChannel:SetParam( {u32UsrFrmDepth=30} )
			if not r then
				_dbg.Error( "AIN", err )
			end
		-- end

		if features == "HI3521D" and self.aiPubAttr.enSoundmode == "MONO" then
			-- self.aiChannel:SetVqeAttr(self.vqeAttr)
			-- self.aiChannel:EnableVqe()
			-- self.VqeEnable = true
		end
	end
end

function channel_meta:Stop()
	if self.initialized then
		self.started = false
		self.aiChannel:Disable()
	end
end

function channel_meta:SelectSource(src)
	if src ~= self.aiSource then
		self.aiSource = src
		if _M.aCodec then
			_M.aCodec:Open()
			if 0 ~= _M.aCodec:SelectSource( self.aiSource ) then
				_dbg.Error( "AIN", "Fail to select audio source:", src )
			end
			_M.aCodec:Close()
		end
	end
end

function channel_meta:SetGain( gain )
	if _M.aCodec then
		_M.aCodec:Open()
		--[[
		if 0 ~= _M.aCodec:SetInputVolume(gain) then
			_dbg.Error( "AIN", "Fail to set audio input gain to:", gain )
		end
		]]
		--_M.aCodec:SetADCGain( gain, gain )
		_M.aCodec:SetMICGain( gain, gain ) -- This is analog gain, so I think it's best.
		_M.aCodec:Close()
	end
end

function channel_meta:Mute()
	if _M.aCodec then
		_M.aCodec:Open()
		_M.aCodec:SetMICMute( true, true )
		_M.aCodec:Close()
	end
end

function channel_meta:Unmute()
	if _M.aCodec then
		_M.aCodec:Open()
		_M.aCodec:SetMICMute( false, false )
		_M.aCodec:Close()
	end
end

channel_meta.__index = channel_meta

---------------------------------------------------

function _M.CreateChannel(id, config)
	if id < 1 or id > _M.MAX_CHANNELS then
		_dbg.Error( "AIN", "Now audio in only support", _M.MAX_CHANNELS, "channels, but you request create channel ", id )
		return nil
	end

	if _M.channels[id] then
		return _M.channels[id]
	end

	config = config or {}

	local chanObj = {
		id = id,
		aiDev = AI.GetDevice(id-1),
		aiChannel = nil,
		started = false,

		aiSource = "LINEIN",

		aiPubAttr = {
			enWorkmode = config.enWorkmode or "I2S_SLAVE", --NOTE: 3521D 不能改，硬件决定
			enSamplerate = config.enSamplerate or 48000,
			enBitwidth = config.enBitwidth or 16,
			enSoundmode = config.enSoundmode or "MONO",
			u32EXFlag = config.u32EXFlag or 0,
			u32FrmNum = config.u32FrmNum or 30,
			u32PtNumPerFrm = config.u32PtNumPerFrm or 1024,
			u32ClkChnCnt = config.u32ClkChnCnt or 2,
			u32ChnCnt = config.u32ChnCnt or 2,
			u32ClkSel = config.u32ClkSel or 0
		},
		vqeAttr = {
			bHpfOpen = 0,
			bAecOpen = 0,
			bAnrOpen = 0,
			bRnrOpen = 0,
			bAgcOpen = 1,
			bEqOpen = 0,
			bHdrOpen = 0,
			s32WorkSampleRate = 16000,
			s32FrameSample = 1024,
			enWorkstate = 0,
			stAgcCfg = {
				bUsrMode = 1,
				s8TargetLevel = -2,
				s8NoiseFloor = -40,
				s8MaxGain = 0,
				s8AdjustSpeed = 10,
				s8ImproveSNR = 2,
				s8UseHighPassFilt = 0,
				s8OutputMode = 0,
				s16NoiseSupSwitch = 1,
				s32Reserved = 1,
			}
		},
	}

	if features == "HI3521D" then
		chanObj.aiPubAttr.enWorkmode = "I2S_SLAVE"
	end

	local r = setmetatable( chanObj, channel_meta )
	_M.channels[id] = r

	return r
end

function _M.Init(resetCodec)
	if features == "HI3536" then
		Init_3536(resetCodec)
	elseif features == "HI3521D" then
		Init_3521D()
	end
end

---------------------------------------------------

return _M


