#!/usr/bin/lua
package.path = package.path .. ";/usr/local/bin/network/?.lua;"
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local service = require "coaf.service"
local thread = require "coaf.thread"
local event = require "coaf.event"
local pt = require "pl.pretty"

coaf.ChDir(coaf.GetAppDir(arg))

local withLog = false
for i = 1, #arg do
    if arg[i] == "--log" then
        withLog = true
        break
    end
end

if withLog then
    local roll = require 'coaf.log.rollfile'
    os.execute('/bin/mkdir -p /var/log/')
    local logF = roll.Create('/var/log/nm.log')
    if not logF or not logF:init() then
        _dbg.Error('NETWORK', 'Fail to open log file: /var/log/nm.log, so the log messages should be output to console.')
        withLog = false
    else
        _dbg.AddLogger('networklogfile', logF, nil, nil)
    end
end

coaf.Init("network")
_G.print = function(...)
    _dbg.Info1( "NETWORK", ... )
end

if withLog then
	local stdio = require "coaf.log.stdio"
	stdio.SetDefaultFilterName( "NETWORK" )
	stdio.Start()
end

local type_change = {
    number = tonumber,
    string = tostring
}

local function empty_function() print("empty function") return false end
local function errror_function() print("error function") return false end

function createNewObj(source)
    if not source or type(source) ~= "table" then
        return
    end
    
    return setmetatable({}, {
        __index = function(_, key)
            local func = source[key]
            if not func then
                print("not have this func and return")
                return empty_function
            end
            --TODO:参数校验
            return function(...)
                local arguments, is_dict
                local tbl = {...}
                if type(tbl[1]) == "table" then
                    is_dict = true
                    arguments = tbl[1]
                else
                    arguments = tbl
                end
            
                local rule = source.Rule[key]
                if not rule then
                    print("not this rule and return empty function")
                    return empty_function()
                end
            
                if not rule.is_open then
                    --TODO: 这里应该区别是网络请求还是内部coaf调用，如果是网络请求就return
                    print("not open this function: ", key)
                    return empty_function()
                end
            
                local args_list = {}
                for i, v in ipairs(rule.parameters) do
                    local value
                    if arguments[v.key] ~= nil then
                        value = arguments[v.key]
                    else
                        value = arguments[i]
                    end
                    if v.must and not value then
                        print("not have the parameters : ", v.key)
                        return errror_function()
                    end
            
                    if value ~= nil then
                        local change_func = type_change[v.type]
                        if change_func then
                            value = change_func(value)
                        end
                        if v.type and type(value) ~= v.type  then
                            print("parameters type is not allowed : ", v.key)
                            return errror_function()
                        end
                    end
            
                    args_list[i] = value
                end
            
                if is_dict then
                    return source[key](table.unpack(args_list, 1, #rule.parameters))
                else
                    return source[key](...)
                end
            end
        end,
    })
end

function table.update(old, new)
    for i, v in pairs(old) do
        if type(new[i]) == "table" and type(v) == "table" then
            table.update(v, new[i])
        elseif type(v) ~= "table" and new[i] ~= nil then
            old[i] = new[i]
        end
    end
end

function table.copy(src)
    if type(src) ~= "table" then
        return
    end

    local new = {}
    for k, v in pairs(src) do
        if type(v) == "table" then
            new[k] = table.copy(v)
        else
            new[k] = v
        end
    end
    return new
end

function table.index(tbl, value)
    for index, v in pairs(tbl) do
        if type(value) == type(v) and value == v then
            return index
        end
    end
end

service.InitSocket()

function split(str, sep)
    local sep, fields = sep or "\t", {}
    local pattern = string.format("([^%s]+)", sep)
    str:gsub(pattern, function(c) fields[#fields+1] = c end)
    return fields
end
-- _G.support_networkManager = true

local network
if _G.support_networkManager then
    network = require "network.networkManager.nm_service"
else
    network = require "network.simpleNetwork.nm_service"
end

local function Main(args)
    os.execute("/sbin/ifconfig lo up 127.0.0.1")
    local coservice = service.New("network", "unix:/var/run/network.sock")
    network.Init(coservice)
    if coservice then
        coservice:CreateCoServerProxy()
        coservice:ExposeObject("/network", createNewObj(network))
    else
        print("~~~~~~~~~Can not create service~~~~~~~~~~")
    end
end

coaf.Start(Main, args)

network.Exit()
