local coaf = require "coaf.core"
local run_daemon = require "coaf.run_daemon"

local _M = {}

function _M.LoadDrivers()
	--os.execute( "/sbin/insmod /hitoe/stmmac.ko hitoe=1" )
	os.execute( "/sbin/insmod /hitoe/stmmac.ko" ) --<< as default.
	run_daemon.Init()
	os.execute( "/bin/ln -sf /usr/sbin/dnsmasq /usr/sbin/dhcp_server" )
	os.execute( "/usr/bin/killall dhcp_server" )
	os.execute( "/usr/bin/killall avahi-daemon" )
end

function _M.RemoveDrivers()
end

function _M.PatchConfigs(nm_config)
	return nm_config
end

function _M.BeforeReset(nmService)
end

function _M.AfterReset(nmService)
end

function _M.FixEnumeratedInterfaces(nmService,intfs)
	return intfs
end

function _M.StartAvahiDaemon(ignoreAddrs)
	--TODO: (this is the new feature): If you configured to enable
	--      avahi-daemon (mDNS) service, after network interfaces
	--      are configured, it will call this function to start 
	--      avahi-daemon. So you can run that service program by
	--      yourself; or if you do nothing, the avahi-daemon 
	--      service is ignored.
	--
	--      'ignoreAddrs' is an array contains all ignored IP addresses
	--      for avahi-daemon service. Typically, these addresses
	--      are fail-safe addresses (alias addresses).
	
	if _M.avahi_daemon_task then
		_M.avahi_daemon_task:Kill()
		_M.avahi_daemon_task:Destroy()
		_M.avahi_daemon_task = nil
	end

	local args = {}
	if ignoreAddrs then
		local i
		for i=1, #ignoreAddrs do
			table.insert( args, "-B" )
			table.insert( args, ignoreAddrs[i] )
		end
	end

	_M.avahi_daemon_task = run_daemon.Create({
		name = "avahi-daemon",
		run = "/usr/sbin/avahi-daemon",
		args = args,
		on_terminate = "restart"
	})

	if _M.avahi_daemon_task then
		_M.avahi_daemon_task:Start()
	end
end

function _M.StopAvahiDaemon()
	--TODO: (this is the new feature): Call this to stop avahi-daemon
	--      (mDNS) service.
	if _M.avahi_daemon_task then
		_M.avahi_daemon_task:Kill()
		_M.avahi_daemon_task:Destroy()
		_M.avahi_daemon_task = nil
	end
end

function _M.GetTools()
	return {
		IFCONFIG = "/sbin/ifconfig",
		ROUTE = "/sbin/route",
		IP = "/bin/ip",
		DHCLIENT = "/sbin/dhclient"
	}
end

return _M

