local _M = {}

local AllAudioSources = {
	[1] = {
		token = "000",
		Channels = 2
	},
	[2] = {
		token = "001",
		Channels = 2
	}
}

local AllVideoSources = {
	[1] = {
		token = "000",
		Framerate = 60,
		Resolution = { Width = 1920, Height = 1080 },
		Imaging = { Brightness = 0, ColorSaturation = 0, Contrast = 0 },
	},
	[2] = {
		token = "001",
		Framerate = 60,
		Resolution = { Width = 1920, Height = 1080 },
		Imaging = { Brightness = 0, ColorSaturation = 0, Contrast = 0 },
	}
}

local AllVideoSourceConfigs = {
	[1] = {
		token = "000",
		Name = "VideoSourceInput_Main",
		UseCount = 1,
		SourceToken = "000",
		Bounds = {
			x = 0, y = 0, width=1920, height=1080
		},
		Extension = {
			Rotate = {
				Mode = tt__RotateMode__OFF, Degree = 0
			}
		}
	},

	[2] = {
		token = "001",
		Name = "VideoSourceInput_Sub",
		UseCount = 1,
		SourceToken = "001",
		Bounds = {
			x = 0, y = 0, width=1920, height=1080
		},
		Extension = {
			Rotate = {
				Mode = tt__RotateMode__OFF, Degree = 0
			}
		}
	}
}

local AllAudioSourceConfigs = {
	[1] = {
		token = "000",
		Name = "AudioSourceInput_Main",
		UseCount = 1,
		SourceToken = "000",
	},
	
	[2] = {
		token = "001",
		Name = "AudioSourceInput_Sub",
		UseCount = 1,
		SourceToken = "001",
	}
}

local GetCoAFProxy = require "GetCoAFProxy"
----------------------------------------------------------
function _M.UpdateAVSourcesAndConfigs()
	print("@@@@@@@@@@@@ in function UpdateAVSourcesAndConfigs, in file profiles_dualsrc.lua")
	-- local signals = busOprs.GetSignalStatus()
	local proxy = GetCoAFProxy("/capture", "codec")
	--NOTE: 获取视频源的信息
	-- if signals then
	if proxy then
		local is_ok, info = proxy.GetInfo(1)
		proxy:Destroy()
		if is_ok then
			AllVideoSources[1].Framerate = math.floor(info.sourceFrameRate) or 0
			if AllVideoSources[1].Framerate <= 0 then
				AllVideoSources[1].Framerate = 50 --TODO: Here.
			end
			
			local width, height = info.sourceSize:match("(%d+)x(%d+)")
			AllVideoSources[1].Resolution = { Width = width or 0, Height = height or 0 }
			AllVideoSources[1].Imaging = { 
				Brightness = info.brightness or 128,
				ColorSaturation = info.saturation or 128,
				Contrast = info.saturation or 128,
			}
			
			AllVideoSourceConfigs[1].Bounds = { x = 0, y = 0, width= width or 0, height = height or 0 }
			
			if info.rotate == 0 then
				AllVideoSourceConfigs[1].Extension.Rotate = { Mode = tt__RotateMode__OFF, Degree = 0 }
			-- elseif rmf == "mirror" or rmf == "flip" then
			-- 	AllVideoSourceConfigs[1].Extension.Rotate = { Mode = tt__RotateMode__AUTO, Degree = (rmf == "flip" and 180 or 0) }
			else
				AllVideoSourceConfigs[1].Extension.Rotate = { Mode = tt__RotateMode__ON, Degree = tonumber(info.rotate) or 0 }
			end
		end
	end
	print("@@@@@@@@@@@@ end of function UpdateAVSourcesAndConfigs, in file profiles_dualsrc.lua")
end
----------------------------------------------------------

local FixedProfiles = {
	[1] = {
		token = "Profile_Main_00",
		Name = "MainStream_Profile",
		fixed = 1,
		
		VideoSourceConfiguration = AllVideoSourceConfigs[1],
		AudioSourceConfiguration = AllAudioSourceConfigs[1],
		
		VideoEncoderConfiguration = {
			Name = "MainVideoEncoder",
			UseCount = 1,
			token = "000",
			Encoding = tt__VideoEncoding__H264,
			
			Resolution = {
				Width = 1920,
				Height = 1080
			},
			
			Quality = 5,
			
			RateControl = { 
				FrameRateLimit = 30,
				EncodingInterval = 1,
				BitrateLimit = 2000
			},

			H264 = {
				GovLength = 30,
				H264Profile = tt__H264Profile__High
			},

			SessionTimeout = 0x7fffffff,
		},
		
		AudioEncoderConfiguration = {
			Name = "MainAudioEncoder",
			UseCount = 1,
			token = "000",
			Encoding = tt__AudioEncoding__AAC,
			Bitrate = 64000,
			SampleRate = 48000,
			Multicast = {
				Address = {
					Type = tt__IPType__IPv4,
					IPv4Address = "224.0.0.1"
				},
				Port = 60000,
				TTL = 127,
				AutoStart = 0,
			},
			
			SessionTimeout = 0x7fffffff,
		},
	},
	
	[2] = {
		token = "Profile_Sub_00",
		Name = "SubStream_Profile",
		fixed = 1,
		
		VideoSourceConfiguration = AllVideoSourceConfigs[2],
		AudioSourceConfiguration = AllAudioSourceConfigs[2],
		
		VideoEncoderConfiguration = {
			Name = "SubVideoEncoder",
			UseCount = 1,
			token = "001",
			Encoding = tt__VideoEncoding__H264,
			
			Resolution = {
				Width = 960,
				Height = 540
			},
			
			Quality = 5,
			
			RateControl = { 
				FrameRateLimit = 30,
				EncodingInterval = 1,
				BitrateLimit = 512
			},

			H264 = {
				GovLength = 30,
				H264Profile = tt__H264Profile__High
			},

			SessionTimeout = 0x7fffffff,
		},
		
		AudioEncoderConfiguration = {
			Name = "SubAudioEncoder",
			UseCount = 1,
			token = "001",
			Encoding = tt__AudioEncoding__AAC,
			Bitrate = 64000,
			SampleRate = 48000,
			Multicast = {
				Address = {
					Type = tt__IPType__IPv4,
					IPv4Address = "224.0.0.1"
				},
				Port = 60002,
				TTL = 127,
				AutoStart = 0,
			},
			
			SessionTimeout = 0x7fffffff,
		},
	}
}


local profile_index = {tt__H264Profile__Baseline, tt__H264Profile__Main, tt__H264Profile__High}
function _M.UpdateAVEncoderConfigs(spec)
	print("@@@@@@@@@@@@@  in function UpdateAVEncoderConfigs, in file profiles_dualsrc.lua")

	local streamList
	if not spec then
		streamList = {"main", "sub"}
	else
		streamList = {spec == 1 and "main" or "sub"}
	end

	local proxy = GetCoAFProxy("/codec", "codec")
	if proxy then
		local is_ok, info = proxy.GetInfo()
		if is_ok then
			for idx, stream in ipairs(streamList) do
				local tbl = FixedProfiles[idx]
				local data = info[stream]
				if data.type == "H264" then
					tbl.VideoEncoderConfiguration.Encoding = tt__VideoEncoding__H264
					tbl.VideoEncoderConfiguration.H264.GovLength = data.rc.u32Gop
					tbl.VideoEncoderConfiguration.H264.H264Profile = profile_index[data.ve.u32Profile + 1]
				end
				
				tbl.VideoEncoderConfiguration.Resolution = {
					Width = data.ve.u32PicWidth == 1 and data.capSize[1] or data.ve.u32PicWidth,
					Height = data.ve.u32PicHeight == 1 and data.capSize[2] or data.ve.u32PicHeight
				}

				tbl.VideoEncoderConfiguration.RateControl.FrameRateLimit = data.rc.fr32DstFrmRate
				tbl.VideoEncoderConfiguration.RateControl.EncodingInterval = 0
				tbl.VideoEncoderConfiguration.RateControl.BitrateLimit = data.rc.mode == "VBR" and data.rc.u32MaxBitRate or data.rc.u32BitRate
				tbl.VideoEncoderConfiguration.Quality = 50 --TODO: S2没有这参数

				--NOTE: 临时参数
				tbl.AudioEncoderConfiguration.Encoding = tt__AudioEncoding__AAC
				tbl.AudioEncoderConfiguration.Bitrate = 64000
				tbl.AudioEncoderConfiguration.SampleRate = 48000
			end
		end
		proxy:Destroy()
	end

	--TODO: S2的音频结构与这里冲突
	-- proxy = GetCoAFProxy("/audio", "codec")
	-- if proxy then
	-- 	local is_ok, info = proxy.GetInfo(1)
	-- 	if is_ok then
	-- 	end
	-- 	proxy:Destroy()
	-- end
	print("@@@@@@@@@@@@@  in function UpdateAVEncoderConfigs, in file profiles_dualsrc.lua")
end

--[==[ *ABSTRACT* The 'response' arguments:
{
	__sizeProfiles: <int>
	Profiles[...]: 
	{
		Name: <char*> (!)
		VideoSourceConfiguration: 
		{
			Name: <char*> (!)
			UseCount: <int> (!)
			token: <char*> (!)
			SourceToken: <char*> (!)
			Bounds: (!)
			{
				x: <int> (!)
				y: <int> (!)
				width: <int> (!)
				height: <int> (!)
			}
			__size: <int>
			__any: <_XML>
			Extension: 
			{
				Rotate: 
				{
					Mode: (!)
					(
						tt__RotateMode__OFF: 0,
						tt__RotateMode__ON: 1,
						tt__RotateMode__AUTO: 2,
					)
					Degree: <int*>
					Extension: 
					{
						__size: <int>
						__any: <_XML>
					}
					__anyAttribute: <_XML>
				}
				Extension: 
				{
					__size: <int>
					__any: <_XML>
				}
			}
			__anyAttribute: <_XML>
		}
		AudioSourceConfiguration: 
		{
			Name: <char*> (!)
			UseCount: <int> (!)
			token: <char*> (!)
			SourceToken: <char*> (!)
			__size: <int>
			__any: <_XML>
			__anyAttribute: <_XML>
		}
		VideoEncoderConfiguration: 
		{
			Name: <char*> (!)
			UseCount: <int> (!)
			token: <char*> (!)
			Encoding: (!)
			(
				tt__VideoEncoding__JPEG: 0,
				tt__VideoEncoding__MPEG4: 1,
				tt__VideoEncoding__H264: 2,
			)
			Resolution: (!)
			{
				Width: <int> (!)
				Height: <int> (!)
			}
			Quality: <float> (!)
			RateControl: 
			{
				FrameRateLimit: <int> (!)
				EncodingInterval: <int> (!)
				BitrateLimit: <int> (!)
			}
			MPEG4: 
			{
				GovLength: <int> (!)
				Mpeg4Profile: (!)
				(
					tt__Mpeg4Profile__SP: 0,
					tt__Mpeg4Profile__ASP: 1,
				)
			}
			H264: 
			{
				GovLength: <int> (!)
				H264Profile: (!)
				(
					tt__H264Profile__Baseline: 0,
					tt__H264Profile__Main: 1,
					tt__H264Profile__Extended: 2,
					tt__H264Profile__High: 3,
				)
			}
			Multicast: (!)
			{
				Address: (!)
				{
					Type: (!)
					(
						tt__IPType__IPv4: 0,
						tt__IPType__IPv6: 1,
					)
					IPv4Address: <char*>
					IPv6Address: <char*>
				}
				Port: <int> (!)
				TTL: <int> (!)
				AutoStart: (!)
				(
					xsd__boolean__false_: 0,
					xsd__boolean__true_: 1,
				)
				__size: <int>
				__any: <_XML>
				__anyAttribute: <_XML>
			}
			SessionTimeout: <LONG64> (!)
			__size: <int>
			__any: <_XML>
			__anyAttribute: <_XML>
		}
		AudioEncoderConfiguration: 
		{
			Name: <char*> (!)
			UseCount: <int> (!)
			token: <char*> (!)
			Encoding: (!)
			(
				tt__AudioEncoding__G711: 0,
				tt__AudioEncoding__G726: 1,
				tt__AudioEncoding__AAC: 2,
			)
			Bitrate: <int> (!)
			SampleRate: <int> (!)
			Multicast: (!)
			{
				Address: (!)
				{
					Type: (!)
					(
						tt__IPType__IPv4: 0,
						tt__IPType__IPv6: 1,
					)
					IPv4Address: <char*>
					IPv6Address: <char*>
				}
				Port: <int> (!)
				TTL: <int> (!)
				AutoStart: (!)
				(
					xsd__boolean__false_: 0,
					xsd__boolean__true_: 1,
				)
				__size: <int>
				__any: <_XML>
				__anyAttribute: <_XML>
			}
			SessionTimeout: <LONG64> (!)
			__size: <int>
			__any: <_XML>
			__anyAttribute: <_XML>
		}
		VideoAnalyticsConfiguration: 
		{
			Name: <char*> (!)
			UseCount: <int> (!)
			token: <char*> (!)
			AnalyticsEngineConfiguration: (!)
			{
				__sizeAnalyticsModule: <int>
				AnalyticsModule[...]: 
				{
					Parameters: (!)
					{
						__sizeSimpleItem: <int>
						SimpleItem[...]: 
						{
							Name: <char*> (!)
							Value: <char*> (!)
						}
						__sizeElementItem: <int>
						ElementItem[...]: 
						{
							__any: <_XML>
							Name: <char*> (!)
						}
						Extension: 
						{
							__size: <int>
							__any: <_XML>
						}
						__anyAttribute: <_XML>
					}
					Name: <char*> (!)
					Type: <_QName> (!)
				}
				Extension: 
				{
					__size: <int>
					__any: <_XML>
				}
				__anyAttribute: <_XML>
			}
			RuleEngineConfiguration: (!)
			{
				__sizeRule: <int>
				Rule[...]: 
				{
					Parameters: (!)
					{
						__sizeSimpleItem: <int>
						SimpleItem[...]: 
						{
							Name: <char*> (!)
							Value: <char*> (!)
						}
						__sizeElementItem: <int>
						ElementItem[...]: 
						{
							__any: <_XML>
							Name: <char*> (!)
						}
						Extension: 
						{
							__size: <int>
							__any: <_XML>
						}
						__anyAttribute: <_XML>
					}
					Name: <char*> (!)
					Type: <_QName> (!)
				}
				Extension: 
				{
					__size: <int>
					__any: <_XML>
				}
				__anyAttribute: <_XML>
			}
			__size: <int>
			__any: <_XML>
			__anyAttribute: <_XML>
		}
		PTZConfiguration: 
		{
			Name: <char*> (!)
			UseCount: <int> (!)
			token: <char*> (!)
			NodeToken: <char*> (!)
			DefaultAbsolutePantTiltPositionSpace: <char*>
			DefaultAbsoluteZoomPositionSpace: <char*>
			DefaultRelativePanTiltTranslationSpace: <char*>
			DefaultRelativeZoomTranslationSpace: <char*>
			DefaultContinuousPanTiltVelocitySpace: <char*>
			DefaultContinuousZoomVelocitySpace: <char*>
			DefaultPTZSpeed: 
			{
				PanTilt: 
				{
					x: <float> (!)
					y: <float> (!)
					space: <char*>
				}
				Zoom: 
				{
					x: <float> (!)
					space: <char*>
				}
			}
			DefaultPTZTimeout: <LONG64*>
			PanTiltLimits: 
			{
				Range: (!)
				{
					URI: <char*> (!)
					XRange: (!)
					{
						Min: <float> (!)
						Max: <float> (!)
					}
					YRange: (!)
					{
						Min: <float> (!)
						Max: <float> (!)
					}
				}
			}
			ZoomLimits: 
			{
				Range: (!)
				{
					URI: <char*> (!)
					XRange: (!)
					{
						Min: <float> (!)
						Max: <float> (!)
					}
				}
			}
			Extension: 
			{
				__size: <int>
				__any: <_XML>
				PTControlDirection: 
				{
					EFlip: 
					{
						Mode: (!)
						(
							tt__EFlipMode__OFF: 0,
							tt__EFlipMode__ON: 1,
							tt__EFlipMode__Extended: 2,
						)
						__size: <int>
						__any: <_XML>
						__anyAttribute: <_XML>
					}
					Reverse: 
					{
						Mode: (!)
						(
							tt__ReverseMode__OFF: 0,
							tt__ReverseMode__ON: 1,
							tt__ReverseMode__AUTO: 2,
							tt__ReverseMode__Extended: 3,
						)
						__size: <int>
						__any: <_XML>
						__anyAttribute: <_XML>
					}
					Extension: 
					{
						__size: <int>
						__any: <_XML>
						__anyAttribute: <_XML>
					}
					__anyAttribute: <_XML>
				}
				Extension: 
				{
					__size: <int>
					__any: <_XML>
				}
			}
			__anyAttribute: <_XML>
		}
		MetadataConfiguration: 
		{
			Name: <char*> (!)
			UseCount: <int> (!)
			token: <char*> (!)
			PTZStatus: 
			{
				Status: (!)
				(
					xsd__boolean__false_: 0,
					xsd__boolean__true_: 1,
				)
				Position: (!)
				(
					xsd__boolean__false_: 0,
					xsd__boolean__true_: 1,
				)
				__anyAttribute: <_XML>
			}
			Events: 
			{
				Filter: 
				{
					__size: <int>
					__any: <_XML>
				}
				SubscriptionPolicy: 
				{
					__size: <int>
					__any: <_XML>
				}
				__size: <int>
				__any: <_XML>
				__anyAttribute: <_XML>
			}
			Analytics: 
			(
				xsd__boolean__false_: 0,
				xsd__boolean__true_: 1,
			)
			Multicast: (!)
			{
				Address: (!)
				{
					Type: (!)
					(
						tt__IPType__IPv4: 0,
						tt__IPType__IPv6: 1,
					)
					IPv4Address: <char*>
					IPv6Address: <char*>
				}
				Port: <int> (!)
				TTL: <int> (!)
				AutoStart: (!)
				(
					xsd__boolean__false_: 0,
					xsd__boolean__true_: 1,
				)
				__size: <int>
				__any: <_XML>
				__anyAttribute: <_XML>
			}
			SessionTimeout: <LONG64> (!)
			__size: <int>
			__any: <_XML>
			AnalyticsEngineConfiguration: 
			{
				__sizeAnalyticsModule: <int>
				AnalyticsModule[...]: 
				{
					Parameters: (!)
					{
						__sizeSimpleItem: <int>
						SimpleItem[...]: 
						{
							Name: <char*> (!)
							Value: <char*> (!)
						}
						__sizeElementItem: <int>
						ElementItem[...]: 
						{
							__any: <_XML>
							Name: <char*> (!)
						}
						Extension: 
						{
							__size: <int>
							__any: <_XML>
						}
						__anyAttribute: <_XML>
					}
					Name: <char*> (!)
					Type: <_QName> (!)
				}
				Extension: 
				{
					__size: <int>
					__any: <_XML>
				}
				__anyAttribute: <_XML>
			}
			Extension: 
			{
				__size: <int>
				__any: <_XML>
			}
			__anyAttribute: <_XML>
		}
		Extension: 
		{
			__size: <int>
			__any: <_XML>
			AudioOutputConfiguration: 
			{
				Name: <char*> (!)
				UseCount: <int> (!)
				token: <char*> (!)
				OutputToken: <char*> (!)
				SendPrimacy: <char*>
				OutputLevel: <int> (!)
				__size: <int>
				__any: <_XML>
				__anyAttribute: <_XML>
			}
			AudioDecoderConfiguration: 
			{
				Name: <char*> (!)
				UseCount: <int> (!)
				token: <char*> (!)
				__size: <int>
				__any: <_XML>
				__anyAttribute: <_XML>
			}
			Extension: 
			{
				__size: <int>
				__any: <_XML>
			}
			__anyAttribute: <_XML>
		}
		token: <char*> (!)
		fixed: 
		(
			xsd__boolean__false_: 0,
			xsd__boolean__true_: 1,
		)
		__anyAttribute: <_XML>
	}
}
]==]

-----------------------------------------

function _M.GetAllProfiles()
	local ptzConfigs = require "ptzConfigs"
	local i
	for i=1, #FixedProfiles do
		FixedProfiles[i].PTZConfiguration = ptzConfigs.GetConfig("000")
	end
	_M.UpdateAVSourcesAndConfigs()
	_M.UpdateAVEncoderConfigs()

	return FixedProfiles
end

-----------------------------------------

function _M.GetProfile(token)
	local ptzConfigs = require "ptzConfigs"

	local i
	for i=1, #FixedProfiles do
		if FixedProfiles[i].token == token then
			FixedProfiles[i].PTZConfiguration = ptzConfigs.GetConfig("000")
			_M.UpdateAVSourcesAndConfigs()
			_M.UpdateAVEncoderConfigs(i)
			return FixedProfiles[i]
		end
	end
	return nil
end

-----------------------------------------
function _M.GetAllVideoSources()
	_M.UpdateAVSourcesAndConfigs()
	return AllVideoSources
end

-----------------------------------------
function _M.GetAllAudioSources()
	_M.UpdateAVSourcesAndConfigs()
	return AllAudioSources
end

-----------------------------------------
function _M.GetAllVideoSourceConfigs()
	_M.UpdateAVSourcesAndConfigs()
	return AllVideoSourceConfigs
end

-----------------------------------------
function _M.GetVideoSourceConfig(token)
	local i
	_M.UpdateAVSourcesAndConfigs()
	for i=1, #AllVideoSourceConfigs do
		if AllVideoSourceConfigs[i].token == token then
			return AllVideoSourceConfigs[i]
		end
	end
	return nil
end

-----------------------------------------
function _M.GetAllAudioSourceConfigs()
	_M.UpdateAVSourcesAndConfigs()
	return AllAudioSourceConfigs
end

-----------------------------------------
function _M.GetAudioSourceConfig(token)
	local i
	_M.UpdateAVSourcesAndConfigs()
	for i=1, #AllAudioSourceConfigs do
		if AllAudioSourceConfigs[i].token == token then
			return AllAudioSourceConfigs[i]
		end
	end
	return nil
end

-----------------------------------------
return _M

