local DC = require "supertable.declare"
local CONF = require "supertable.core"
local _M = {
	config_file = "/data/configs/record.conf"
}

local function setRecordStart(owner,pObj,value)
	local diskRecorder = require "diskRecorder"
	pObj:InternalSetInt(value)
	pObj:InternalSetChanged(false)

	if _G.CONFIG_LOADING then
		return
	end

	local streamId = owner:Name()
	if tonumber(value) ~= 0 then
		diskRecorder.StartRecord( streamId )
	else
		diskRecorder.StopRecord( streamId )
	end
end

local function onRecordParamsChanged(obj)
	if obj.start ~= 0 then	--Is started, so try update recording arguments.
		local streamId = obj:Name()
		diskRecorder.StartRecord( streamId )
	end
end

local RECORD_MAN_Template = DC.SET(
	DC.OBJ{
		auto_record = DC.BOOL(true),
		start = DC.BOOL(false) : __nosave(true),
		disk_type = DC.STR("largest"),
		format = DC.STR("ts"),
		file_prefix = DC.STR("REC"),
		limit_type = DC.STR("size-loop"),
		limit_size = DC.NUM(4000000),
		limit_time = DC.NUM(3600),
		disk_policy = DC.STR("overwrite"),
		choose = DC.BOOL(true),
		bindAudio = DC.INT(1),
	}
)	: __changed( function(conf)
	conf:TAB_SAVE()
end)


local default = {
	[0] = {
		auto_record = true,
		start = false,
		disk_type = "largest",
		format = "ts",
		file_prefix = "REC",
		limit_type = "size-loop",
		limit_size = 4000000,
		limit_time = 3600,
		disk_policy = "overwrite",
		choose = true,
		bindAudio = 1,
	},
	[1] = {
		auto_record = true,
		start = false,
		disk_type = "largest",
		format = "ts",
		file_prefix = "REC",
		limit_type = "size-loop",
		limit_size = 4000000,
		limit_time = 3600,
		disk_policy = "overwrite",
		choose = true,
		bindAudio = 1,
	},
}

function _M.Load()
	local conf = CONF.Create({}, RECORD_MAN_Template)
	conf:TAB_STORAGE( "file", { file =  _M.config_file } )
	local _, r = conf:TAB_LOAD()
    if not r then
		conf:TAB_BIND( default )
    end
	conf:TAB_RESET_CHANGE()
	return conf
end

return _M
