#!/usr/bin/env lua
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

-- Change current dir to app location
coaf.ChDir(coaf.GetAppDir(arg))

local socket = require "cosocket.core"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"

local roll = require "coaf.log.rollfile"

local withLog = false
local _x
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

if withLog then
	-- ** initialize the log file **
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /tmp/" )
	local logF = roll.Create("/tmp/record_manager.log")
	if not logF or not logF:init() then
		_dbg.Error("RECORD_M", "Fail to open log file: /tmp/record_manager.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("logfile", logF, nil, nil)
	end
	-- ** log **
end

coaf.Init( "record_manager" )

if withLog then
	local stdio = require "coaf.log.stdio"
	stdio.SetDefaultFilterName( "RECORD" )
	stdio.Start()
end

_G.print = function(...)
	_dbg.Info1( "RECORD_M", ... )
end

local diskRecorder = require "diskRecorder"

---------------------------------------------------
_G.record_m = require "config".Load()
---------------------------------------------------

service.InitSocket()

local cosvc_Record
local period_timer

local function diskCheck()
	diskRecorder.StepCheck()
end

function table.update(old, new)
    for i, v in pairs(old) do
        if type(new[i]) == "table" and type(v) == "table" then
            table.update(v, new[i])
        elseif type(v) ~= "table" and new[i] ~= nil then
            old[i] = new[i]
        end
    end
end

local type_change = {
    number = tonumber,
    string = tostring
}
local function empty_function() print("empty function") return true end
local function errror_function() print("error function") return false end
local function createResult(...)
    local result = {...}
    local is_ok = result[1]
    if not is_ok then
        print("###########XPCALL-ERROR#######", result[2])
        return
    end
    return table.unpack(result, 2, #result)
end

local function createNewObj(source, name)
    if not source or type(source) ~= "table" then
        return
    end
    
    return setmetatable({}, {
        __index = function(_, key)
            local func = source[key]
            if not func then
                print("not have this func and return", name)
                return empty_function
            end

            return function(...)
                local arguments, is_dict
                local tbl = {...}
                if type(tbl[1]) == "table" then
                    is_dict = true
                    arguments = tbl[1]
                else
                    arguments = tbl
                end
            
                local rule = source.Rule[key]
                -- print("key(function name) = ", key, name)
                if not rule then
                    print("not this rule and return empty function", name)
                    return empty_function()
                end
            
                if not rule.is_open then
                    --NOTE: 这里应该区别是网络请求还是内部coaf调用，如果是网络请求就return
                    print("not open this function: ", key, name)
                    return empty_function()
                end
            
                local args_list = {}
                for i, v in ipairs(rule.parameters) do
                    local value
                    if arguments[v.key] ~= nil then
                        value = arguments[v.key]
                    else
                        value = arguments[i]
                    end
                    if v.must and not value then
                        print("not have the parameters : ", v.key, name)
                        return errror_function()
                    end
            
                    if value ~= nil then
                        local change_func = type_change[v.type]
                        if change_func then
                            value = change_func(value)
                        end
                        if type(value) ~= v.type  then
                            print("parameters type is not allowed : ", v.key, key, name)
                            return errror_function()
                        end
                    end
            
                    args_list[i] = value
                end
            
                if is_dict then
                    return createResult(xpcall(function()
                        return source[key](table.unpack(args_list, 1, #rule.parameters))
                    end, debug.traceback))
                else
                    return createResult(xpcall(function()
                        return source[key](table.unpack(tbl, 1, #tbl))
                    end, debug.traceback))
                end
            end
        end,
    })
end

local function Main()

-- NOTE here:
-- The record manager is some specical compares to other service.
-- When 'udev' found removable disk(s), it should call 'recordUdevClients'
-- and 'recordUdevClients' will lauch this service by DBus. At that 
-- moment, the 'coserver' maybe not started yet! (udev is started 
-- at system loading time). So, the co-service creation should be
-- fail, need try it again and again. Now the service module support
-- the retrying.
    diskRecorder.Init()

	cosvc_Record = service.New( "record", "unix:/var/run/record.sock" )
	if cosvc_Record then
		_dbg.Info( "RECORD_M", "CoService of 'record' is created." )
		cosvc_Record:CreateCoServerProxy()
	else
		_dbg.Error( "RECORD_M", "Fail created CoService!" )
	end

	if cosvc_Record then
		cosvc_Record:ExposeObject( "/record", createNewObj(diskRecorder, "diskRecorder") )
	end

	print( "Disk/recording manager service running..." )

	period_timer = coaf.NewTimer( diskCheck, 0.01, 5 )--After 0.01 second, and per 5 seconds check.
	if period_timer then
		period_timer:start()
	end
end

coaf.Start(Main)

--------
_dbg.Info( "RECORD_M", "RecordManager service is terminated." )


