require "features"
require "linuxUsers"

local _dbg = require "coaf.debug"

local NGINX = "/usr/local/openresty/nginx/sbin/nginx"
local NGINX_CONF_LOC = "/usr/local/openresty/nginx/conf"
local NGINX_CONF_TEMPLATE = "nginx.conf.default"
local FLAG_HTTP_PORT = "@HTTP_PORT@"
local FLAG_HTTPS_PORT = "@HTTPS_PORT@"
-- local FLAG_ONVIF_PORT = "@ONVIF_PORT@"

local NGINX_CERT_KEY = "server.key"
local NGINX_CERT_CRT = "server.crt"

local cfg_users = require("users_config").Load()

local _M = {}

local function StartNginx()
	local lfs = require "lfs"
	--Install cert
	if not lfs.attributes( NGINX_CONF_LOC .. "/" .. NGINX_CERT_KEY ) then
		os.execute( "/bin/cp -Rf " .. NGINX_CONF_LOC .. "/" .. NGINX_CERT_KEY .. ".default " .. NGINX_CONF_LOC .. "/" .. NGINX_CERT_KEY )
		os.execute( "/bin/sync" )
	else
		local f = io.open( NGINX_CONF_LOC .. "/" .. NGINX_CERT_KEY, "r" )
		local valid
		if f then
			local line = f:read("*l")
			f:close()
			if line and line:match("%-BEGIN") then
				valid = true
			end
		end
		if not valid then
			os.execute( "/bin/cp -Rf " .. NGINX_CONF_LOC .. "/" .. NGINX_CERT_KEY .. ".default " .. NGINX_CONF_LOC .. "/" .. NGINX_CERT_KEY )
			os.execute( "/bin/sync" )
		end
	end
	if not lfs.attributes( NGINX_CONF_LOC .. "/" .. NGINX_CERT_CRT ) then
		os.execute( "/bin/cp -Rf " .. NGINX_CONF_LOC .. "/" .. NGINX_CERT_CRT .. ".default " .. NGINX_CONF_LOC .. "/" .. NGINX_CERT_CRT )
		os.execute( "/bin/sync" )
	else
		local f = io.open( NGINX_CONF_LOC .. "/" .. NGINX_CERT_CRT, "r" )
		local valid
		if f then
			local line = f:read("*l")
			f:close()
			if line and line:match("%-BEGIN") then
				valid = true
			end
		end
		if not valid then
			os.execute( "/bin/cp -Rf " .. NGINX_CONF_LOC .. "/" .. NGINX_CERT_CRT .. ".default " .. NGINX_CONF_LOC .. "/" .. NGINX_CERT_CRT )
			os.execute( "/bin/sync" )
		end
	end
	os.execute( NGINX ) --Start nginx web server.
end

local function ReStartNginx()
    os.execute(NGINX .. " -s reload")
end

function _M.UpdateNginx(cfg)
    if cfg then
        if not tonumber(cfg.port) or not tonumber(cfg.https_port) then
            return false
        end
        cfg_users.webDaemon.port = tonumber(cfg.port)
        cfg_users.webDaemon.https_port = tonumber(cfg.https_port)
        cfg_users:__TRIGGER(true)
    end

	local is_ok = os.execute( "/bin/sed -e 's/" .. FLAG_HTTP_PORT .. "/" .. cfg_users.webDaemon.port .. "/g' " .. 
				"-e 's/" .. FLAG_HTTPS_PORT .. "/" .. cfg_users.webDaemon.https_port .. "/g' " ..
				-- "-e 's/" .. FLAG_ONVIF_PORT .. "/" .. onvif_port .. "/g' " .. 
				NGINX_CONF_LOC .. "/" .. NGINX_CONF_TEMPLATE .. " > " .. NGINX_CONF_LOC .. "/nginx.conf" )
    if is_ok then
        ReStartNginx()
    end
    return true
end

function _M.GetInfo()
    return {
        port = cfg_users.webDaemon.port,
        https_port = cfg_users.webDaemon.https_port
    }
end


function _M.Init()
    _M.UpdateNginx()
    StartNginx()
end

function _M.GetHostname()
	-- if _M.OSType() == "Windows" then
	-- 	return nil
	-- end

	local r
	local f = io.popen( "/bin/hostname", "r" )
	if f then
		r = f:read("*l")
		f:close()
	end
	if not r then
		f = io.open( "/etc/hostname", "r" )
		if f then
			r = f:read("*l")
			f:close()
		end
	end
	if not r or r == "" then
		return "none"
	else
		return r
	end
end

function _M.SetHostname(name)
	-- if _M.OSType() == "Windows" then
	-- 	return nil, "Unsupported hostname set in Windows"
	-- end

	if name then
		local f = io.open( "/etc/hostname", "w" )
		if f then
			f:write(name .. "\n")
			f:close()
		end
		os.execute( "/bin/hostname '" .. name .. "'" )
		return name
	else
		return nil, "Invalid host name"
	end
end

return _M