#!/usr/bin/env lua
local coaf = require 'coaf.core'
local _dbg = require 'coaf.debug'

coaf.ChDir(coaf.GetAppDir(arg))

local socket = require 'cosocket.core'
local client = require 'coaf.client'
local adapter = require 'coaf.cli_adapter.simple'
local service = require 'coaf.service'
local websocket_server = require 'websocket_server'

--NOTE: 需要配置nginx转发websocket请求的超时时间
local withLog = false
local _x
for _x = 1, #arg do
    if arg[_x] == '--log' then
        withLog = true
        break
    end
end

if withLog then
    local roll = require 'coaf.log.rollfile'
    os.execute('/bin/mkdir -p /var/log/')
    local logF = roll.Create('/var/log/websocket.log')
    if not logF or not logF:init() then
        _dbg.Error('WEBSOCKET', 'Fail to open log file: /var/log/websocket.log, so the log messages should be output to console.')
    else
        _dbg.AddLogger('logfile', logF)
    end
end

function get_contents(filename)
    local f = io.open(filename, "rb")
    local contents = ""
    if f then
      contents = f:read "*a"
      f:close()
    end
  
    return contents
  end

coaf.Init('webSocketSvc')
service.InitSocket()
client.InitSocket(socket, adapter)

local websocket_coaf = require 'websocket_coaf'

local function Main(args)
    websocket_server.Init()
    websocket_coaf.Init()
    local cosvc_websocket = service.New('webSocketSvc', 'unix:/var/run/webSocketSvc.sock')
    if cosvc_websocket then
        _dbg.Info('WEBSOCKET', "CoService of 'webSocketSvc' is created.")
        cosvc_websocket:CreateCoServerProxy()
        cosvc_websocket:ExposeObject('/websocket', websocket_coaf)
    else
        _dbg.Error('WEBSOCKET', 'Fail created CoService!')
    end
end

coaf.Start(Main, arg)
_dbg.Print('WEBSOCKET', 'Service terminated.')